# -*- coding:utf-8 -*-

from django.contrib import admin
from coop_local.models import Event, EventCategory, Calendar, Occurrence
from coop.utils.autocomplete_admin import FkAutocompleteAdmin, InlineAutocompleteAdmin

#from genericadmin.admin import GenericAdminModelAdmin
# GenericStackedInline or GenericTabularInline
# on fera le lien generique depuis l'objet en question si besoin (article, événement)


class EventCategoryAdmin(admin.ModelAdmin):
    list_display = ('label',)


class OccurrenceInline(admin.TabularInline):
    model = Occurrence
    extra = 1


class EventAdmin(FkAutocompleteAdmin):
    #content_type_blacklist = ('auth/group', 'auth/user', )
    list_display = ('title', 'event_type', 'description')
    list_filter = ('event_type', )
    search_fields = ('title', 'description')
    related_search_fields = {'person': ('last_name', 'first_name',
                                        'email', 'structure', 'username'),
                            'organization': ('title', 'acronym', 'subtitle', 'description')
    }
    fieldsets = ((None, {'fields': (
        'title', 'description',
        ('event_type', 'calendar'),
        ('organization', 'person'),
        #'remote_organization_label'
      )
                            }),)

    inlines = [OccurrenceInline, ]

admin.site.register(Event, EventAdmin)
admin.site.register(EventCategory, EventCategoryAdmin)
admin.site.register(Calendar)



import datetime

# A "strftime" string for formatting start and end time selectors in forms
TIMESLOT_TIME_FORMAT = '%H:%M'

# Used for creating start and end time form selectors as well as time slot grids.
# Value should be datetime.timedelta value representing the incremental 
# differences between temporal options
TIMESLOT_INTERVAL = datetime.timedelta(minutes=15)

# A datetime.time value indicting the starting time for time slot grids and form
# selectors
TIMESLOT_START_TIME = datetime.time(9)

# A datetime.timedelta value indicating the offset value from 
# TIMESLOT_START_TIME for creating time slot grids and form selectors. The for
# using a time delta is that it possible to span dates. For instance, one could
# have a starting time of 3pm (15:00) and wish to indicate a ending value 
# 1:30am (01:30), in which case a value of datetime.timedelta(hours=10.5) 
# could be specified to indicate that the 1:30 represents the following date's
# time and not the current date.
TIMESLOT_END_TIME_DURATION = datetime.timedelta(hours=+8)

# Indicates a minimum value for the number grid columns to be shown in the time
# slot table.
TIMESLOT_MIN_COLUMNS = 4

# Indicate the default length in time for a new occurrence, specifed by using
# a datetime.timedelta object
DEFAULT_OCCURRENCE_DURATION = datetime.timedelta(hours=+1)

# If not None, passed to the calendar module's setfirstweekday function.
CALENDAR_FIRST_WEEKDAY = 6



# -*- coding:utf-8 -*-

# TODO : est-ce qu'on peut proposer des valeurs par defaut ici ?
# ou deriver d'index pre-existants et leur ajouter quelques champs secifiques au site ?

import datetime
from haystack import indexes, site
from coop_local.models import Organization


class OrganizationIndex(indexes.RealTimeSearchIndex):
    text = indexes.CharField(document=True, use_template=True)
    title = indexes.CharField(model_attr='title')
    acronym = indexes.CharField(model_attr='subtitle')
    description = indexes.CharField(model_attr='description')
    tags = indexes.MultiValueField()

    def prepare_tags(self, obj):
        return [tag.name for tag in obj.tags.all()]

site.register(Organization, OrganizationIndex)


# -*- coding:utf-8 -*-

from django.contrib import admin
from coop_local.models import *
from coop.person.admin import PersonAdmin
from coop.org.admin import OrganizationAdmin
from django.conf import settings

# -- Loading base models

admin.site.register(LinkProperty)

admin.site.register(Role)
admin.site.register(PersonCategory)
admin.site.register(Person, PersonAdmin)

admin.site.register(Organization, OrganizationAdmin)
admin.site.register(OrganizationCategory)

if "coop.exchange" in settings.INSTALLED_APPS:
    from coop.exchange.admin import ExchangeAdmin
    admin.site.register(ExchangeMethod)
    admin.site.register(Exchange, ExchangeAdmin)

if "coop_cms" in settings.INSTALLED_APPS:
    from coop.article.admin import CoopArticleAdmin
    from coop_cms.settings import get_article_class
    article_model = get_article_class()
    admin.site.unregister(article_model)
    admin.site.register(article_model, CoopArticleAdmin)

if "coop_tag" in settings.INSTALLED_APPS:
    from coop_tag.settings import get_class
    admin.site.unregister(get_class('tag'))
    admin.site.register(get_class('tag'))

if 'forms_builder.forms' in settings.INSTALLED_APPS:
    from coop.admin import CoopFormAdmin
    from forms_builder.forms.models import Form
    admin.site.unregister(Form)
    admin.site.register(Form, CoopFormAdmin)



"""
This file was generated with the custommenu management command, it contains
the classes for the admin menu, you can customize this class as you want.

To activate your custom menu add the following to your settings.py::
    ADMIN_TOOLS_MENU = 'devcoop.menu.CustomMenu'
"""

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from admin_tools.menu import items, Menu


class CustomMenu(Menu):
    """
    Custom Menu for devcoop admin site.
    """
    def __init__(self, **kwargs):
        Menu.__init__(self, **kwargs)
        self.children = [
            #items.MenuItem(_('Dashboard'), reverse('admin:index')),
            #items.Bookmarks('Favoris'),

            items.MenuItem(_('Navigation tree'), '/admin/coop_local/navtree/1/', icon='icon-list-alt icon-white'),

            items.MenuItem(_('Articles'), '/admin/coop_local/article/', icon='icon-pencil icon-white'),

            items.MenuItem(_('CMS'), '#', icon='icon-cog icon-white',
                children=[

                    items.MenuItem(_('Content'), '#', icon='icon-file', children=[
                        items.MenuItem(_('Article categories'), '/admin/coop_cms/articlecategory/'),
                        items.MenuItem(_('Documents'), '/admin/coop_cms/document/'),
                        items.MenuItem(_('Images'), '/admin/coop_cms/image/'),
                        items.MenuItem(_('Newsletters'), '/admin/coop_cms/newsletter/'),
                        items.MenuItem(_('Comments'), '/admin/comments/comment/'),
                        ]),

                    # RSS Sync menu gets inserted here if installed (see above)

                    items.MenuItem(_('Tags'), '#', icon='icon-tags', children=[
                        items.MenuItem(_('Tags'), '/admin/coop_local/tag/'),
                        #items.MenuItem(_('Tag categories'), '/admin/coop_tag/tagcategory/'),
                        items.MenuItem(_('Tag trees'), '/admin/coop_local/navtree/'),
                        ]),

                    items.MenuItem(_('Django'), '#', icon='icon-coop icon-django', children=[
                        items.MenuItem(_('Users'), '/admin/auth/user/'),
                        items.MenuItem(_('Sites'), '/admin/sites/site/'),
                        ]),
                ]
            ),


            # Agenda menu inserted here if coop.agenda installed

            items.MenuItem(_('Network'), '#', icon='icon-coop icon-group icon-white',
                children=[

                    items.MenuItem(_('Directory'), '#', icon='icon-home', children=[
                        items.MenuItem(_('Organizations'), '/admin/coop_local/organization/'),
                        items.MenuItem(_('Persons'), '/admin/coop_local/person/'),
                        items.MenuItem(_('Organization categories'), '/admin/coop_local/organizationcategory/'),
                        items.MenuItem(_('Person categories'), '/admin/coop_local/personcategory/'),
                        items.MenuItem(_('Roles'), '/admin/coop_local/role/'),
                        ]),

                    items.MenuItem(_('Exchanges'), '#', icon='icon-random', children=[
                        items.MenuItem(_('Exchanges'), '/admin/coop_local/exchange/'),
                        items.MenuItem(_('Exchange methods'), '/admin/coop_local/exchangemethod/'),
                        ]),

                    items.MenuItem(_('Cartography'), '#', icon='icon-map-marker', children=[
                        items.MenuItem(_('Locations'), '/admin/coop_geo/location/'),
                        items.MenuItem(_('Areas'), '/admin/coop_geo/area/'),
                        items.MenuItem(_('Location categories'), '/admin/coop_geo/locationcategory/'),
                        # create my map !
                        ]),

                    items.MenuItem(_('RDF settings'), '#', icon='icon-coop icon-rdf', children=[
                        items.MenuItem(_('URI redirection'), '/admin/uriredirect/'),
                        # webid
                        ]),


                ]
            ),



        ]

        if 'coop_cms.apps.rss_sync' in  settings.INSTALLED_APPS:
            self.children[2].children.insert(1,

                    items.MenuItem(_('RSS'), '#', icon='icon-coop icon-rss', children=[
                        items.MenuItem(_('RSS items'), '/admin/rss_sync/rssitem/'),
                        items.MenuItem(_('RSS sources'), '/admin/rss_sync/rsssource/'),
                        ])
                    )

        if 'coop.agenda' in settings.INSTALLED_APPS:
            self.children.insert(2,
                items.MenuItem(_('Agenda'), '#', icon='icon-calendar icon-white',
                    children=[
                        items.MenuItem(_('Events'), '/admin/coop_local/event/'),
                        items.MenuItem(_('Calendar'), '/admin/coop_local/calendar/'),
                        items.MenuItem(_('Event categories'), '/admin/coop_local/eventcategory/'),
                    ])
                )

    def init_with_context(self, context):
        """
        Use this method if you need to access the request context.
        """
        return super(CustomMenu, self).init_with_context(context)



# -*- coding:utf-8 -*-
# Django settings for {{ project_name }} project.

from django.core.exceptions import ImproperlyConfigured

DEBUG = False
TEMPLATE_DEBUG = DEBUG

SECRET_KEY = '{{ secret_key }}'

import os.path
PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
#PROJECT_NAME = PROJECT_PATH.split('/')[-1]
PROJECT_NAME = '{{ project_name }}'  # which one is safer ?


# import all default app settings from django-coop app

try:
    from coop.default_project_settings import *
except ImportError, exp:
    raise ImproperlyConfigured("Unable to find default_project_settings.py file from django-coop : ", exp)

# override settings when needed in local_settings.py

try:
    from local_settings import *
except ImportError, exp:
    pass

# debug settings : load dev tools (FireLogger & Django debug Toolbar) or setup Sentry Logging
try:
    DEBUG_SETTINGS = {  'apps': INSTALLED_APPS,
                        'middleware': MIDDLEWARE_CLASSES,
                        'logging': LOGGING
                        }
    from coop.debug_settings import *
except ImportError, exp:
    raise ImproperlyConfigured("Unable to find coop/debug_settings.py file in django-coop : ", exp)


# db_settings file for DATABASES, and CACHE backend settings

try:
    from db_settings import *
except ImportError, exp:
    raise ImproperlyConfigured("Unable to find db_settings.py file from coop_local project : ", exp)


# common db_settings file for DATABASE ROUTERS

try:
    from coop.db_settings import *
except ImportError, exp:
    raise ImproperlyConfigured("Unable to find coop/db_settings.py file in django-coop : ", exp)





# -*- coding:utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import Site
from django_push.publisher.feeds import Feed
from django.db.models.loading import get_model
from django.contrib.contenttypes.models import ContentType
from coop.signals import LastDTProcess


class UpdateFeed(Feed):
    title = _(u"Updates for %s." % Site.objects.get_current().name)
    link = "/feed/"
    description = _(u"All records updates listed on the %s website." % Site.objects.get_current().name)
    hub = "http://%s/hub/" % Site.objects.get_current()

    description_template = 'feeds/description.html'  
    title_template = 'feeds/title.html'


    def items(self):
        # Call content type pour trouver les models ....
        # TODO : object such as uri_mode = 'common' or 'imported' are also published.
        # Is it the good feature?
        # return get_model(self._mType.app_label, self._model).objects.order_by('-modified')[:5]
        # print 'FeedItem CALLLED'
        qs = get_model(self._mType.app_label, self._model).objects.filter(modified__gte=LastDTProcess.get())
        if len(qs) > 5:
            return qs.order_by('-modified')
        else:
            return get_model(self._mType.app_label, self._model).objects.order_by('-modified')[:5]


    def item_link(self, item):
        return item.uri

    def item_guid(self, item):
        return "%s_%s" % (item.uuid, item.modified)

    def item_pubdate(self, item):
        return item.modified

    def item_categories(self, item):
        return ('django', 'coop')


    # def item_extra_kwargs
    def get_object(self, request, *args, **kwargs):
        self._model = kwargs['model']
        try:
            self._mType = ContentType.objects.get(model=self._model)
        except ContentType.MultipleObjectsReturned:
            self._mType = ContentType.objects.get(model=self._model, app_label='coop_local')


        self.title = _(u"Updates for %s on %s." % (self._model, Site.objects.get_current().name))
        self.link = "/feed/%s/" % self._model
        return None






# -*- coding:utf-8 -*-

import os
from coop_local.settings import PROJECT_PATH, PROJECT_NAME

TIME_ZONE = 'Europe/Paris'
LANGUAGE_CODE = 'fr-FR'
SITE_ID = 1
USE_I18N = True
USE_L10N = True

DEFAULT_CONTENT_TYPE = 'text/html'
DEFAULT_CHARSET = 'utf-8'

LOGIN_URL = '/accounts/login/'
LOGIN_REDIRECT_URL = '/'

USE_TZ = False

# Upload directory
MEDIA_ROOT = os.path.abspath(PROJECT_PATH + '/media/')
MEDIA_URL = '/media/'

# Static files
STATIC_ROOT = os.path.abspath(PROJECT_PATH + '/static_collected/')
STATIC_URL = '/static/'

# compat fix ?
ADMIN_MEDIA_PREFIX = '/static/admin/'


import admin_tools
ADMIN_TOOLS_PATH = os.path.dirname(os.path.abspath(admin_tools.__file__))


STATICFILES_DIRS = [
    os.path.abspath(ADMIN_TOOLS_PATH + '/media/'),
]

STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
]

TEMPLATE_DIRS = [
    os.path.abspath(PROJECT_PATH + '/coop_local/templates/')
]

TEMPLATE_LOADERS = [
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
]

MIDDLEWARE_CLASSES = [
    #'django.middleware.cache.UpdateCacheMiddleware',
    'django.middleware.gzip.GZipMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    #'django.middleware.cache.FetchFromCacheMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django_webid.auth.middleware.WEBIDAuthMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'coop.utils.middleware.CORSMiddleware',
    'pagination.middleware.PaginationMiddleware',
]

TEMPLATE_CONTEXT_PROCESSORS = [
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.request',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.core.context_processors.tz',
    'django.contrib.messages.context_processors.messages',
    'coop.context_processors.current_site',
]

ROOT_URLCONF = 'coop_local.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'coop_local.wsgi.application'

# Cache middelware must be enabled above
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
    },
    'memcached': {
        'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
        #'LOCATION': '127.0.0.1:11211',
        'LOCATION': 'unix:/tmp/memcached.sock',
    }
}

CACHE_MIDDLEWARE_ALIAS = 'memcached'
CACHE_MIDDLEWARE_SECONDS = 60 * 60
CACHE_MIDDLEWARE_KEY_PREFIX = PROJECT_NAME


INSTALLED_APPS = [

    # admin tools
    'admintools_bootstrap',
    'admin_tools',
    'admin_tools.theming',
    'admin_tools.menu',
    'admin_tools.dashboard',

    # django contribs
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    #'django.contrib.admindocs', # ramène sa traduction de Tags "Étiquettes"...
    'django.contrib.gis',
    'django.contrib.comments',

    # other 3rd parties
    'south',
    'django_extensions',
    'extended_choices',
    'floppyforms',
    #'haystack',
    #'oembed',
    'chosen',
    'sorl.thumbnail',
    'tinymce',
    'autoslug',
    'slugify',

    # WebID
    'django_webid.provider',
    'django_webid.auth',

    # coop apps
    'coop_tag',

    # 'feincms',  # for their MPTT tree editor, not synced
    'coop',  # override feincms tree editor template
    'coop.link',
    'coop.org',
    'coop.person',
    'coop.ui',
    'coop.webid',

    #'coop_geo',

    # PuSH
    'django_rq',
    'subhub',
    'django_push.subscriber',
    'uriredirect',

    # coop_cms
    'djaloha',
    'coop_cms',
    'coop_cms.apps.rss_sync',
    'colorbox',
    'coop_bar',
    'pagination',

]


AUTOSLUG_SLUGIFY_FUNCTION = 'slugify.slugify'

TAG_MODEL = 'coop_local.models.Tag'
TAGGEDITEM_MODEL = 'coop_local.models.TaggedItem'

TAG_MODEL_FKEY_NAME = 'coop_local.Tag'

TAGGER_CSS_FILENAME = 'coop_tag.css'

THUMBNAIL_FORMAT = 'PNG'

TINYMCE_DEFAULT_CONFIG = {
    'theme': "advanced",
    'relative_urls': False,
    'width': '617px', 'height': '220px',
    'theme_advanced_toolbar_location': 'top',
    'theme_advanced_buttons1': 'bold,italic,|,justifyleft,justifycenter,justifyright,|,bullist,numlist,|,link,unlink,|,code',
    'theme_advanced_buttons2': '', 'theme_advanced_buttons3': ''
    }

COOP_CMS_ARTICLE_CLASS = 'coop_local.models.Article'
COOP_CMS_ARTICLE_FORM = 'coop.forms.ArticleForm'
COOP_CMS_ARTICLE_LOGO_SIZE = '600'


COOP_CMS_NAVTREE_CLASS = 'coop_local.NavTree'

# You need to declare the app here to be able to select models from navigable types in coop-cms
COOP_CMS_CONTENT_APPS = ('coop_local', 'coop_tag', 'coop_geo', 'coop_cms')

DJALOHA_LINK_MODELS = (
        'coop_local.Article',
        'coop_cms.ArticleCategory',
        'coop_local.Organization',
        'coop_local.OrganizationCategory',
        'coop_local.Event',
        )



# COOP_CMS_ARTICLE_TEMPLATES = 'coop_local.get_article_templates' # marche plus ?

COOP_CMS_ARTICLE_TEMPLATES = [
    ('coop_cms/article_standard.html', 'Standard'),
    ('coop_cms/article_rubrique.html', 'Rubrique'),
]

COOPBAR_MODULES = [
    'coop_cms.coop_bar_cfg',
    'coop.coop_bar_cfg'
    ]

FORMS_BUILDER_USE_SITES = False

HAYSTACK_SITECONF = 'coop_local.search_sites'
HAYSTACK_SEARCH_ENGINE = 'simple'
HAYSTACK_CONNECTIONS = {
    'default': {
        'ENGINE': 'haystack.backends.simple_backend.SimpleEngine',
    },
}

ADMIN_TOOLS_MENU = 'coop.ui.menu.CustomMenu'
ADMIN_TOOLS_INDEX_DASHBOARD = 'coop.ui.dashboard.CustomIndexDashboard'
ADMIN_TOOLS_APP_INDEX_DASHBOARD = 'coop.ui.dashboard.CustomAppIndexDashboard'
ADMIN_TOOLS_THEMING_CSS = 'css/coop_bootstrap_theming.css'


AUTHENTICATION_BACKENDS = [
    'django_webid.auth.backends.WEBIDAuthBackend',
    'coop_cms.perms_backends.ArticlePermissionBackend',
    'django.contrib.auth.backends.ModelBackend',
]

AUTH_PROFILE_MODULE = 'coop_local.Person'

# projection used in database
COOP_GEO_EPSG_PROJECTION = 4326  # WGS84

COOP_GEO_BOUNDING_BOX = []
COOP_GEO_REGION = LANGUAGE_CODE[:2]



#
# WebID Options
#

WEBIDAUTH_USE_COOKIE = True

# The following lines set the local user creation callback.
# All the info from the remote profile is accessible from
# the request.webidinfo object (see documentation).


def createusercb(req):
    from coop.webid.utils import build_coop_user
    return build_coop_user(req)

WEBIDAUTH_CREATE_USER_CALLBACK = createusercb

WEBIDPROVIDER_SKIP_PROFILE_INIT = True

# Uncomment the following lines if you want to specify
# a custom callback for the assigment of the WebIDUris.
# If this callback is not specified, provider will look for
# a webidprovider-webid_uri urlpattern and will try to
# reverse it passing the webiduser instance.

#def webidcb(webiduser):
#    "avoids circular import"
#    from coop.webid.utils import custom_webid_uri
#    return custom_webid_uri(webiduser)
#WEBIDPROVIDER_WEBIDURI_CALLBACK = webidcb


LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'simplest': {
            'format': '%(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(asctime)s %(message)s'
        },

    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
        'console-dumb': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simplest'
        },

        # access denied problem on prod env.... to be check
        #  'file_subhub': {  # define and name a handler
        #     'level': 'DEBUG',
        #     'formatter': 'simple',
        #     'class': 'logging.handlers.WatchedFileHandler',  # set the logging class to log to a file
        #     'filename': os.path.abspath(PROJECT_PATH + '/logs/subhub.log')  # log file
        # },
        #  'file_rqworker': {  # define and name a handler
        #     'level': 'DEBUG',
        #     'formatter': 'simple',
        #     'class': 'logging.handlers.WatchedFileHandler',  # set the logging class to log to a file
        #     'filename': os.path.abspath(PROJECT_PATH + '/logs/rq.log')  # log file
        # },


    },
    'loggers': {
        'django.db.backends': {
            'level': 'ERROR',
            'handlers': ['console'],
            'propagate': False,
        },
        'subhub.maintenance': {
            'handlers': ['console'],
            'level': 'DEBUG'
        },
        'subhub.distribution.process': {
            'handlers': ['console'],
            'level': 'DEBUG'

        },
        'worker': {
            'handlers': ['console'],
            'level': 'DEBUG'

        },
        'coop': {
            'handlers': ['console-dumb'],
            'level': 'DEBUG',
        },
    },

}

# import default app settings from django-coop app

try:
    from coop.settings import *
except ImportError, exp:
    raise ImproperlyConfigured("Unable to find settings.py file from django-coop")



"""
Common features and functions for agenda
"""
from collections import defaultdict
from datetime import datetime, date, time, timedelta
import itertools

from django.db.models.query import QuerySet
from django.utils.safestring import mark_safe
from dateutil import rrule
from coop.agenda.conf import settings as agenda_settings


def html_mark_safe(func):
    """
    Decorator for functions return strings that should be treated as template
    safe.
    """
    def decorator(*args, **kws):
        return mark_safe(func(*args, **kws))
    return decorator


def time_delta_total_seconds(time_delta):
    """
    Calculate the total number of seconds represented by a
    ``datetime.timedelta`` object
    """
    return time_delta.days * 3600 + time_delta.seconds


def month_boundaries(dt=None):
    """
    Return a 2-tuple containing the datetime instances for the first and last
    dates of the current month or using ``dt`` as a reference.
    """
    import calendar
    dt = dt or date.today()
    wkday, ndays = calendar.monthrange(dt.year, dt.month)
    start = datetime(dt.year, dt.month, 1)
    return (start, start + timedelta(ndays - 1))


def css_class_cycler():
    """
    Return a dictionary keyed by ``EventCategory`` abbreviations, whose values are an
    iterable or cycle of CSS class names.
    """
    from coop_local.models import EventCategory
    return defaultdict(
        lambda: itertools.cycle(('evt-even', 'evt-odd')).next,
        ((e.slug, itertools.cycle((
             'evt-%s-even' % e.slug,
             'evt-%s-odd' % e.slug
             )).next) for e in EventCategory.objects.all()
        )
    )


class BaseOccurrenceProxy(object):
    """
    A simple wrapper class for handling the presentational aspects of an
    ``Occurrence`` instance.
    """
    def __init__(self, occurrence, col):
        self.column = col
        self._occurrence = occurrence
        self.event_class = ''

    def __getattr__(self, name):
        return getattr(self._occurrence, name)

    def __unicode__(self):
        return self.title


class DefaultOccurrenceProxy(BaseOccurrenceProxy):

    def __init__(self, *args, **kws):
        super(DefaultOccurrenceProxy, self).__init__(*args, **kws)
        link = '<a href="%s">%s</a>' % (
            self.get_absolute_url(),
            self.title
        )

        self._str = itertools.chain((link,),itertools.repeat(r'\\\///')).next

    @html_mark_safe
    def __unicode__(self):
        #print self.title
        return self._str()


def create_timeslot_table(dt=None, items=None,
    start_time=agenda_settings.TIMESLOT_START_TIME,
    end_time_delta=agenda_settings.TIMESLOT_END_TIME_DURATION,
    time_delta=agenda_settings.TIMESLOT_INTERVAL,
    min_columns=agenda_settings.TIMESLOT_MIN_COLUMNS,
    css_class_cycles=css_class_cycler,
    proxy_class=DefaultOccurrenceProxy):
    """
    Create a grid-like object representing a sequence of times (rows) and
    columns where cells are either empty or reference a wrapper object for
    event occasions that overlap a specific time slot.

    Currently, there is an assumption that if an occurrence has a ``start_time``
    that falls with the temporal scope of the grid, then that ``start_time`` will
    also match an interval in the sequence of the computed row entries.

    * ``dt`` - a ``datetime.datetime`` instance or ``None`` to default to now
    * ``items`` - a queryset or sequence of ``Occurrence`` instances. If
      ``None``, default to the daily occurrences for ``dt``
    * ``start_time`` - a ``datetime.time`` instance
    * ``end_time_delta`` - a ``datetime.timedelta`` instance
    * ``time_delta`` - a ``datetime.timedelta`` instance
    * ``min_column`` - the minimum number of columns to show in the table
    * ``css_class_cycles`` - if not ``None``, a callable returning a dictionary
      keyed by desired ``EventCategory`` abbreviations with values that iterate over
      progressive CSS class names for the particular abbreviation.
    * ``proxy_class`` - a wrapper class for accessing an ``Occurrence`` object.
      This class should also expose ``event_type`` and ``event_type`` attrs, and
      handle the custom output via its __unicode__ method.
    """
    from coop_local.models import Occurrence
    dt = dt or datetime.now()
    dtstart = datetime.combine(dt.date(), start_time)
    dtend = dtstart + end_time_delta

    if isinstance(items, QuerySet):
        items = items._clone()
    elif not items:
        items = Occurrence.objects.daily_occurrences(dt).select_related('event')

    # build a mapping of timeslot "buckets"
    timeslots = dict()
    n = dtstart
    while n <= dtend:
        timeslots[n] = {}
        n += time_delta

    # fill the timeslot buckets with occurrence proxies
    for item in sorted(items):
        if item.end_time <= dtstart:
            # this item began before the start of our schedle constraints
            continue

        if item.start_time > dtstart:
            rowkey = current = item.start_time
        else:
            rowkey = current = dtstart

        timeslot = timeslots.get(rowkey, None)
        if timeslot is None:
            # TODO fix atypical interval boundry spans
            # This is rather draconian, we should probably try to find a better
            # way to indicate that this item actually occurred between 2 intervals
            # and to account for the fact that this item may be spanning cells
            # but on weird intervals
            continue

        colkey = 0
        while 1:
            # keep searching for an open column to place this occurrence
            if colkey not in timeslot:
                proxy = proxy_class(item, colkey)
                timeslot[colkey] = proxy

                while current < item.end_time:
                    rowkey = current
                    row = timeslots.get(rowkey, None)
                    if row is None:
                        break

                    # we might want to put a sanity check in here to ensure that
                    # we aren't trampling some other entry, but by virtue of
                    # sorting all occurrence that shouldn't happen
                    row[colkey] = proxy
                    current += time_delta
                break

            colkey += 1

    # determine the number of timeslot columns we should show
    column_lens = [len(x) for x in timeslots.itervalues()]
    column_count = max((min_columns, max(column_lens) if column_lens else 0))
    column_range = range(column_count)
    empty_columns = ['' for x in column_range]

    if css_class_cycles:
        column_classes = dict([(i, css_class_cycles()) for i in column_range])
    else:
        column_classes = None

    # create the chronological grid layout
    table = []
    for rowkey in sorted(timeslots.keys()):
        cols = empty_columns[:]
        for colkey in timeslots[rowkey]:
            proxy = timeslots[rowkey][colkey]
            cols[colkey] = proxy
            if not proxy.event_class and column_classes:
                proxy.event_class = column_classes[colkey][proxy.event_type.abbr]()

        table.append((rowkey, cols))

    return table



# -*- coding:utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.conf import settings


if 'coop_tag' in settings.INSTALLED_APPS:
    from coop_tag.models import TagBase, GenericTaggedItemBase, TaggedItemBase
    from coop.models import URIModel
    # in coop_settings, TAG_MODEL = 'coop_local.Tag'

    class CoopTaggedItem(GenericTaggedItemBase, TaggedItemBase):
        #tag = models.ForeignKey('coop_local.Tag', related_name="%(app_label)s_%(class)s_taggeditem_items")

        class Meta:
            verbose_name = _(u'tagged item')
            verbose_name_plural = _(u'tagged items')
            abstract = True
            app_label = "coop_local"

    class CoopTag(TagBase, URIModel):
        # Fields name and slug are defined in TagBase

        language = models.CharField(_(u'language'), max_length=10, default='fr')
        person = models.ForeignKey('coop_local.Person', blank=True, null=True, verbose_name=_(u'person'), editable=False)
        person_uri = models.CharField(_(u'person URI'), blank=True, max_length=250, editable=False)

        # Thesaurus link
        concept_uri = models.CharField(_(u'Concept URI'), blank=True, max_length=250, editable=False)

        # Tags have a common uri domain
        domain_name = 'data.economie-solidaire.fr'

        @property
        def uri_id(self):
            return self.slug

        def label(self):
            return self.name

        def get_absolute_url(self):
            return reverse('tag_detail', args=[self.slug])

        class Meta:
            verbose_name = _(u'tag')
            verbose_name_plural = _(u'tags')
            abstract = True
            app_label = "coop_local"







#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup, find_packages
except ImportError:
    import ez_setup
    ez_setup.use_setuptools()
    from setuptools import setup, find_packages

VERSION = __import__('coop').__version__

import os


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = 'django-coop',
    version = VERSION,
    description = 'A basis for a cooperative organization directory, with people, organization, offers.',
    packages = ['coop',
                'coop.bin',
                'coop.management',
                'coop.management.commands',
                'coop.article',
                'coop.exchange',
                'coop.link',
                'coop.org',
                'coop.mailing',
                'coop.agenda',
                'coop.agenda.conf',
                'coop.person',
                'coop.tag',
                'coop.templatetags',
                'coop.utils',
                'coop.webid',
                'coop.ui',
                ],
    include_package_data = True,
    author = 'Cooperative Quinode',
    author_email = 'contact@quinode.fr',
    license = 'BSD',
    zip_safe = False,
    install_requires = ['south', 'django',
                        #'sorl-thumbnail==11.09',
                        #'django-extensions==0.8',  # waiting for pull request, git://github.com/quinode/django-extensions.git@use_admin.jquery
                        #'django-admin-tools==0.4.1', # for 1.4 : https://bitbucket.org/psyton/django-admin-tools
                        #'django-haystack==1.2.6',
                        #'djangoembed==0.1.1', # plutot via github: https://github.com/ericflo/django-oembed
                        ],
    long_description = open('README.rst').read(),
    url = 'https://github.com/quinode/django-coop/',
    download_url = 'https://github.com/quinode/django-coop/tarball/master',
    scripts = [
        'coop/bin/coop-admin.py',
        'coop/bin/runinenv.sh'],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Natural Language :: English',
        'Natural Language :: French',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
    ],

)



# -*- coding:utf-8 -*-

# WARNING : This is bootstraping code , DO NOT CHANGE ANYTHING
# DO NOT add custom models here , add them in local_models.py and they'll be loaded

import sys
from coop.utils.loading import load_models
from coop import base_models
from coop_local.models import local_models

main_module = sys.modules[__name__]

load_models(base_models, local_models, main_module)

# TaggableManager is only available when TaggedItem is loaded
# so we add tags fields after loading all the coop_local models

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

if "coop_tag" in settings.INSTALLED_APPS:
    from coop_tag.managers import TaggableManager
    # from coop_local.models import TaggedItem
    t = TaggableManager(through=TaggedItem,
            blank=True, verbose_name=_(u'Tags'),
            help_text="Une liste de tags avec des virgules")
    t.contribute_to_class(Organization, "tags")
    t.contribute_to_class(Person, "tags")
    t.contribute_to_class(Article, "tags")
    if "coop.exchange" in settings.INSTALLED_APPS:
        t.contribute_to_class(Exchange, "tags")






#!/usr/bin/env python
import os
# import sys

from django.core import management
# from coop import management as coop_management
import coop.management

if __name__ == "__main__":
    # global management._commands
    if management._commands is None:
        management._commands = dict([(name, 'coop') for name in management.find_commands(coop.management.__path__[0])])
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "coop.minimal_settings")


    management.execute_from_command_line()



# -*- coding:utf-8 -*-

from django.conf import settings

# Here you can override any settings from coop default settings files
# See :
# - coop/default_project_settings.py
# - coop/db_settings.py

SITE_AUTHOR = 'Organisme'
SITE_TITLE = 'Demo Django-coop'
DEFAULT_URI_DOMAIN = '{{ domain }}'

DEBUG = False
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    ('Administrateur', 'web@quinode.fr'),
)

MANAGERS = ADMINS
SEND_BROKEN_LINK_EMAILS = True
INTERNAL_IPS = ('127.0.0.1', '92.243.30.98')

SUBHUB_MAINTENANCE_AUTO = False    # set this value to True to automatically syncronize with agregator
PES_HOST = 'pes.domain.com'
THESAURUS_HOST = 'thess.domain.com'

INSTALLED_APPS = settings.INSTALLED_APPS + [
    # select your coop components
    'coop.tag',
    'coop.agenda',
    'coop.article',
    'coop.mailing',
    'coop.exchange',
    #'coop.webid',
    'coop_local',

    # coop optional modules
    'coop_geo',  # est obligatoirement placé APRES coop_local
]